/****************************************************************************
** Meta object code from reading C++ file 'KPropertySet.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kproperty-3.0.1/src/KPropertySet.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KPropertySet.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_KPropertySet_t {
    QByteArrayData data[13];
    char stringdata0[182];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KPropertySet_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KPropertySet_t qt_meta_stringdata_KPropertySet = {
    {
QT_MOC_LITERAL(0, 0, 12), // "KPropertySet"
QT_MOC_LITERAL(1, 13, 15), // "propertyChanged"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 13), // "KPropertySet&"
QT_MOC_LITERAL(4, 44, 3), // "set"
QT_MOC_LITERAL(5, 48, 10), // "KProperty&"
QT_MOC_LITERAL(6, 59, 8), // "property"
QT_MOC_LITERAL(7, 68, 23), // "propertyChangedInternal"
QT_MOC_LITERAL(8, 92, 13), // "propertyReset"
QT_MOC_LITERAL(9, 106, 21), // "aboutToDeleteProperty"
QT_MOC_LITERAL(10, 128, 16), // "aboutToBeCleared"
QT_MOC_LITERAL(11, 145, 16), // "aboutToBeDeleted"
QT_MOC_LITERAL(12, 162, 19) // "readOnlyFlagChanged"

    },
    "KPropertySet\0propertyChanged\0\0"
    "KPropertySet&\0set\0KProperty&\0property\0"
    "propertyChangedInternal\0propertyReset\0"
    "aboutToDeleteProperty\0aboutToBeCleared\0"
    "aboutToBeDeleted\0readOnlyFlagChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KPropertySet[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   49,    2, 0x06 /* Public */,
       7,    2,   54,    2, 0x06 /* Public */,
       8,    2,   59,    2, 0x06 /* Public */,
       9,    2,   64,    2, 0x06 /* Public */,
      10,    0,   69,    2, 0x06 /* Public */,
      11,    0,   70,    2, 0x06 /* Public */,
      12,    0,   71,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    4,    6,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    4,    6,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    4,    6,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    4,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void KPropertySet::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KPropertySet *_t = static_cast<KPropertySet *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->propertyChanged((*reinterpret_cast< KPropertySet(*)>(_a[1])),(*reinterpret_cast< KProperty(*)>(_a[2]))); break;
        case 1: _t->propertyChangedInternal((*reinterpret_cast< KPropertySet(*)>(_a[1])),(*reinterpret_cast< KProperty(*)>(_a[2]))); break;
        case 2: _t->propertyReset((*reinterpret_cast< KPropertySet(*)>(_a[1])),(*reinterpret_cast< KProperty(*)>(_a[2]))); break;
        case 3: _t->aboutToDeleteProperty((*reinterpret_cast< KPropertySet(*)>(_a[1])),(*reinterpret_cast< KProperty(*)>(_a[2]))); break;
        case 4: _t->aboutToBeCleared(); break;
        case 5: _t->aboutToBeDeleted(); break;
        case 6: _t->readOnlyFlagChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (KPropertySet::*_t)(KPropertySet & , KProperty & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KPropertySet::propertyChanged)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (KPropertySet::*_t)(KPropertySet & , KProperty & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KPropertySet::propertyChangedInternal)) {
                *result = 1;
                return;
            }
        }
        {
            typedef void (KPropertySet::*_t)(KPropertySet & , KProperty & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KPropertySet::propertyReset)) {
                *result = 2;
                return;
            }
        }
        {
            typedef void (KPropertySet::*_t)(KPropertySet & , KProperty & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KPropertySet::aboutToDeleteProperty)) {
                *result = 3;
                return;
            }
        }
        {
            typedef void (KPropertySet::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KPropertySet::aboutToBeCleared)) {
                *result = 4;
                return;
            }
        }
        {
            typedef void (KPropertySet::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KPropertySet::aboutToBeDeleted)) {
                *result = 5;
                return;
            }
        }
        {
            typedef void (KPropertySet::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KPropertySet::readOnlyFlagChanged)) {
                *result = 6;
                return;
            }
        }
    }
}

const QMetaObject KPropertySet::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_KPropertySet.data,
      qt_meta_data_KPropertySet,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KPropertySet::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPropertySet::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KPropertySet.stringdata0))
        return static_cast<void*>(const_cast< KPropertySet*>(this));
    return QObject::qt_metacast(_clname);
}

int KPropertySet::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void KPropertySet::propertyChanged(KPropertySet & _t1, KProperty & _t2)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KPropertySet::propertyChangedInternal(KPropertySet & _t1, KProperty & _t2)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void KPropertySet::propertyReset(KPropertySet & _t1, KProperty & _t2)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void KPropertySet::aboutToDeleteProperty(KPropertySet & _t1, KProperty & _t2)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void KPropertySet::aboutToBeCleared()
{
    QMetaObject::activate(this, &staticMetaObject, 4, Q_NULLPTR);
}

// SIGNAL 5
void KPropertySet::aboutToBeDeleted()
{
    QMetaObject::activate(this, &staticMetaObject, 5, Q_NULLPTR);
}

// SIGNAL 6
void KPropertySet::readOnlyFlagChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, Q_NULLPTR);
}
struct qt_meta_stringdata_KPropertyBuffer_t {
    QByteArrayData data[8];
    char stringdata0[87];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KPropertyBuffer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KPropertyBuffer_t qt_meta_stringdata_KPropertyBuffer = {
    {
QT_MOC_LITERAL(0, 0, 15), // "KPropertyBuffer"
QT_MOC_LITERAL(1, 16, 18), // "intersectedChanged"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 13), // "KPropertySet&"
QT_MOC_LITERAL(4, 50, 3), // "set"
QT_MOC_LITERAL(5, 54, 10), // "KProperty&"
QT_MOC_LITERAL(6, 65, 4), // "prop"
QT_MOC_LITERAL(7, 70, 16) // "intersectedReset"

    },
    "KPropertyBuffer\0intersectedChanged\0\0"
    "KPropertySet&\0set\0KProperty&\0prop\0"
    "intersectedReset"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KPropertyBuffer[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x09 /* Protected */,
       7,    2,   29,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    4,    6,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    4,    6,

       0        // eod
};

void KPropertyBuffer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KPropertyBuffer *_t = static_cast<KPropertyBuffer *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->intersectedChanged((*reinterpret_cast< KPropertySet(*)>(_a[1])),(*reinterpret_cast< KProperty(*)>(_a[2]))); break;
        case 1: _t->intersectedReset((*reinterpret_cast< KPropertySet(*)>(_a[1])),(*reinterpret_cast< KProperty(*)>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject KPropertyBuffer::staticMetaObject = {
    { &KPropertySet::staticMetaObject, qt_meta_stringdata_KPropertyBuffer.data,
      qt_meta_data_KPropertyBuffer,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KPropertyBuffer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPropertyBuffer::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KPropertyBuffer.stringdata0))
        return static_cast<void*>(const_cast< KPropertyBuffer*>(this));
    return KPropertySet::qt_metacast(_clname);
}

int KPropertyBuffer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPropertySet::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
QT_END_MOC_NAMESPACE
