/****************************************************************************
** Meta object code from reading C++ file 'KPropertyEditorView.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kproperty-3.0.1/src/KPropertyEditorView.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KPropertyEditorView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_KPropertyEditorView_t {
    QByteArrayData data[29];
    char stringdata0[366];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KPropertyEditorView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KPropertyEditorView_t qt_meta_stringdata_KPropertyEditorView = {
    {
QT_MOC_LITERAL(0, 0, 19), // "KPropertyEditorView"
QT_MOC_LITERAL(1, 20, 18), // "propertySetChanged"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 13), // "KPropertySet*"
QT_MOC_LITERAL(4, 54, 3), // "set"
QT_MOC_LITERAL(5, 58, 9), // "changeSet"
QT_MOC_LITERAL(6, 68, 10), // "SetOptions"
QT_MOC_LITERAL(7, 79, 7), // "options"
QT_MOC_LITERAL(8, 87, 16), // "propertyToSelect"
QT_MOC_LITERAL(9, 104, 11), // "setAutoSync"
QT_MOC_LITERAL(10, 116, 6), // "enable"
QT_MOC_LITERAL(11, 123, 10), // "isAutoSync"
QT_MOC_LITERAL(12, 134, 11), // "acceptInput"
QT_MOC_LITERAL(13, 146, 16), // "setGridLineColor"
QT_MOC_LITERAL(14, 163, 5), // "color"
QT_MOC_LITERAL(15, 169, 14), // "currentChanged"
QT_MOC_LITERAL(16, 184, 7), // "current"
QT_MOC_LITERAL(17, 192, 8), // "previous"
QT_MOC_LITERAL(18, 201, 10), // "commitData"
QT_MOC_LITERAL(19, 212, 8), // "QWidget*"
QT_MOC_LITERAL(20, 221, 6), // "editor"
QT_MOC_LITERAL(21, 228, 20), // "slotSetWillBeCleared"
QT_MOC_LITERAL(22, 249, 20), // "slotSetWillBeDeleted"
QT_MOC_LITERAL(23, 270, 23), // "slotReadOnlyFlagChanged"
QT_MOC_LITERAL(24, 294, 19), // "slotPropertyChanged"
QT_MOC_LITERAL(25, 314, 13), // "KPropertySet&"
QT_MOC_LITERAL(26, 328, 10), // "KProperty&"
QT_MOC_LITERAL(27, 339, 8), // "property"
QT_MOC_LITERAL(28, 348, 17) // "slotPropertyReset"

    },
    "KPropertyEditorView\0propertySetChanged\0"
    "\0KPropertySet*\0set\0changeSet\0SetOptions\0"
    "options\0propertyToSelect\0setAutoSync\0"
    "enable\0isAutoSync\0acceptInput\0"
    "setGridLineColor\0color\0currentChanged\0"
    "current\0previous\0commitData\0QWidget*\0"
    "editor\0slotSetWillBeCleared\0"
    "slotSetWillBeDeleted\0slotReadOnlyFlagChanged\0"
    "slotPropertyChanged\0KPropertySet&\0"
    "KProperty&\0property\0slotPropertyReset"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KPropertyEditorView[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   94,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    2,   97,    2, 0x0a /* Public */,
       5,    1,  102,    2, 0x2a /* Public | MethodCloned */,
       5,    3,  105,    2, 0x0a /* Public */,
       5,    2,  112,    2, 0x2a /* Public | MethodCloned */,
       9,    1,  117,    2, 0x0a /* Public */,
      11,    0,  120,    2, 0x0a /* Public */,
      12,    0,  121,    2, 0x0a /* Public */,
      13,    1,  122,    2, 0x0a /* Public */,
      15,    2,  125,    2, 0x09 /* Protected */,
      18,    1,  130,    2, 0x09 /* Protected */,
      21,    0,  133,    2, 0x09 /* Protected */,
      22,    0,  134,    2, 0x09 /* Protected */,
      23,    0,  135,    2, 0x09 /* Protected */,
      24,    2,  136,    2, 0x09 /* Protected */,
      28,    2,  141,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 6,    4,    7,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QByteArray, 0x80000000 | 6,    4,    8,    7,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QByteArray,    4,    8,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QColor,   14,
    QMetaType::Void, QMetaType::QModelIndex, QMetaType::QModelIndex,   16,   17,
    QMetaType::Void, 0x80000000 | 19,   20,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 25, 0x80000000 | 26,    4,   27,
    QMetaType::Void, 0x80000000 | 25, 0x80000000 | 26,    4,   27,

       0        // eod
};

void KPropertyEditorView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KPropertyEditorView *_t = static_cast<KPropertyEditorView *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->propertySetChanged((*reinterpret_cast< KPropertySet*(*)>(_a[1]))); break;
        case 1: _t->changeSet((*reinterpret_cast< KPropertySet*(*)>(_a[1])),(*reinterpret_cast< SetOptions(*)>(_a[2]))); break;
        case 2: _t->changeSet((*reinterpret_cast< KPropertySet*(*)>(_a[1]))); break;
        case 3: _t->changeSet((*reinterpret_cast< KPropertySet*(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< SetOptions(*)>(_a[3]))); break;
        case 4: _t->changeSet((*reinterpret_cast< KPropertySet*(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        case 5: _t->setAutoSync((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: { bool _r = _t->isAutoSync();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 7: _t->acceptInput(); break;
        case 8: _t->setGridLineColor((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 9: _t->currentChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 10: _t->commitData((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 11: _t->slotSetWillBeCleared(); break;
        case 12: _t->slotSetWillBeDeleted(); break;
        case 13: _t->slotReadOnlyFlagChanged(); break;
        case 14: _t->slotPropertyChanged((*reinterpret_cast< KPropertySet(*)>(_a[1])),(*reinterpret_cast< KProperty(*)>(_a[2]))); break;
        case 15: _t->slotPropertyReset((*reinterpret_cast< KPropertySet(*)>(_a[1])),(*reinterpret_cast< KProperty(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (KPropertyEditorView::*_t)(KPropertySet * );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KPropertyEditorView::propertySetChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

const QMetaObject KPropertyEditorView::staticMetaObject = {
    { &QTreeView::staticMetaObject, qt_meta_stringdata_KPropertyEditorView.data,
      qt_meta_data_KPropertyEditorView,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KPropertyEditorView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPropertyEditorView::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KPropertyEditorView.stringdata0))
        return static_cast<void*>(const_cast< KPropertyEditorView*>(this));
    return QTreeView::qt_metacast(_clname);
}

int KPropertyEditorView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void KPropertyEditorView::propertySetChanged(KPropertySet * _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_END_MOC_NAMESPACE
